# -*- coding: utf-8 -*-
from __future__ import division
import powergama
import powergama.GIS
import powergama.scenarios
import time
import matplotlib.pyplot as plt
import csv

timerange=range(0,8760)

data = powergama.GridData()

datapath= "data/"
resultpath= "results/"
scenarioPrefix = "2014_"
rerun = False
sqlfile = "results_db_.sqlite3"

data.readGridData(nodes=datapath + scenarioPrefix + "nodes.csv",
                  ac_branches=datapath + scenarioPrefix + "branches.csv",
                  dc_branches=datapath + scenarioPrefix+ "hvdc.csv",
                  generators=datapath + scenarioPrefix + "generators.csv",
                  consumers=datapath + scenarioPrefix + "consumers.csv")
data.readProfileData(inflow=datapath+"profiles_inflow.csv",
            demand=datapath+"profiles_demand.csv",
            storagevalue_filling=datapath+"profiles_storval_filling.csv",
            storagevalue_time=datapath+"profiles_storval_time.csv",
            timerange=timerange, 
            timedelta=1.0)


#Update data files based on scenario file:
#powergama.scenarios.newScenario(data,"Scenario_input.csv","data/2014ae_v29_")
# Save present data as scenario file (for modification and reloading):
#powergama.scenarios.saveScenario(data,"2014model_savedScenario_check.csv")

#raise Exception()

lp = powergama.LpProblem(data)
if rerun:
    res = powergama.Results(data,resultpath+sqlfile,replace=True)
    start_time = time.time()
    lp.solve(res)
    end_time = time.time()
    print("\nExecution time = "+str(end_time - start_time)+"seconds")
else:
    res = powergama.Results(data,resultpath+sqlfile,replace=False)
'''
fileName = 'newNTCs.csv'
ac_branches = res.db.getGridInterareaBranches()
dc_branches = res.grid.getDcBranches()

with open(fileName) as f:
    reader = csv.reader(f)
    
    for border in reader:
        from_area = border[0]
        to_area = border[1]
        newCap = int(border[2])
        border_name = from_area + '->' + to_area
        border_branches = {}
        border_branches['ac'] = []
        border_branches['dc'] = []
        oldCap = 0
        newCapCheck = 0           
        
        for branch in ac_branches:
            if branch.count(from_area) + branch.count(to_area) == 2:
                border_branches['ac'].append(branch[0])
                oldCap += data.branch.capacity[branch[0]]
    
        for branch in dc_branches:
            if (branch.count(from_area) < 2) and (branch.count(to_area) < 2):
                if branch.count(from_area) + branch.count(to_area) == 2:
                    border_branches['dc'].append(branch[0])
                    oldCap += data.dcbranch.capacity[branch[0]]
        
        for branch in border_branches['ac']:
            data.branch.capacity[branch] *= (newCap/oldCap)
            newCapCheck += data.branch.capacity[branch]
            
        for branch in border_branches['dc']:
            data.dcbranch.capacity[branch] *= (newCap/oldCap)
            newCapCheck += data.dcbranch.capacity[branch]
        
        print('Border: ' + border_name + ', Old cap: ' + str(oldCap) + ', New cap: ' + str(newCapCheck))

#Update data files based on scenario file:
powergama.scenarios.newScenario(data,"Scenario_input.csv","data/2014ae_v6_")
# Save present data as scenario file (for modification and reloading):
powergama.scenarios.saveScenario(data,"2014model_savedScenario_check.csv")

raise Exception()
'''
#res.writeKeyResultsToExcel(country='MA') #

#powergama.GIS.makekml(res,kmlfile=resultpath+"results.kml",timeMaxMin=None)

#res.plotMapGrid(nodetype="nodalprice",branchtype="",
 #               show_node_labels=False, dotsize=10, draw_par_mer=False
 #               show_Title=False)
#plt.gcf().set_size_inches(7.5,4)
#plt.savefig(resultpath+"2030_map2.pdf", bbox_inches = 'tight')

#res.plotGenerationPerArea('MA',fill=True)
#res.plotEnergyMix(relative=True,showTitle=False)
#plt.gcf().set_size_inches(7.5,4)
#plt.savefig(resultpath+"2030_energymix.pdf", bbox_inches = 'tight')

#res.plotEnergySpilled(gentypes=['wind','wind_offshore','solar_pv','solar_csp','hydro'],showTitle=False)
#plt.gcf().set_size_inches(7.5,4)
#plt.savefig(resultpath+"2030_energyspilled.pdf", bbox_inches = 'tight')

# Plot production from a CSP plant with storage (Andasol, ES, node='E-187')
#genindx=302
#timemaxmin=timerange
#res.plotGeneratorOutput(generator_index=genindx,timeMaxMin=timemaxmin)

# Plot nodal price at associated node
#res.plotStorageValues(genindx=genindx,timeMaxMin=timemaxmin)

# Tune parameters
# Use this to adjust initial filling level for storage generators:
#final_filling = (lp._storage / data.generator.storage).tolist()
#
# Use this to adjust base costs for storage generators:
#mean_area_prices=res.getAreaPricesAverage()

#Investigate switzerland negative price nodes
#res.plotMapGrid(nodetype="nodalprice",branchtype="utilisation",
  #              dotsize=30,timeMaxMin=[3000,4000],filter_node=[-15,15],
  ###              latlon=[43,5,48,12],show_node_labels=True)          
